@@ -44,6 +44,7 @@
 #include "entry.h"
 #include "mm.h"
 #include "panic.h"
+#include "vramwrite.h"
 
 #define MAXNUM_OF_SYSMEMMAP		256
 
@@ -239,33 +240,65 @@ u32 vmm_term_inf()
         return vmm_start_phys+VMMSIZE_ALL ;
 }
 
+#if 0
+void
+dump_pt(char *name, void *page_table)
+{
+	u64 *pt = page_table;
+	int i;
+
+	printf("%s %p\n", name, pt);
+	for (i = 0; i < PTE_NUM_IN_TABLE_64 / 4; i++) {
+		printf("    %016llx %016llx %016llx %016llx\n",
+		       pt[0], pt[1], pt[2], pt[3]);
+		pt += 4;
+	}
+}
+#endif
+
 static void
 move_vmm (void)
 {
 	int i;
 	ulong cr3;
 
-	/* map memory areas copied to at 0xC0000000 */
+	/* Create a page directory to map vmm area */
 	for (i = 0; i < VMMSIZE_ALL >> PAGESIZE2M_SHIFT; i++)
-		vmm_pd[i] =
+		vmm_pd1[i] =
 			(vmm_start_phys + (i << PAGESIZE2M_SHIFT)) |
 			PDE_P_BIT | PDE_RW_BIT | PDE_PS_BIT | PDE_A_BIT |
 			PDE_D_BIT;
-	entry_pdp[3] = (((u64)(ulong)vmm_pd) - 0x40000000) | PDPE_ATTR;
+	/* map vmm area at 0xC0000000 temporarily */
+	entry_pdp[3] = (((u64)(ulong)vmm_pd1) - 0x40000000) | PDPE_ATTR;
 	asm_rdcr3 (&cr3);
 	asm_wrcr3 (cr3);
 
-	/* make a new page directory */
-	vmm_base_cr3 = sym_to_phys (vmm_pdp);
-	memcpy (vmm_pd1, entry_pd0, PAGESIZE);
+	/* make a new page directory for switching from/to real mode */
+
+	/* virt 0x00000000 identity mapping */
 	vmm_pdp[0] = sym_to_phys (entry_pd0) | PDPE_ATTR;
-	vmm_pdp[1] = sym_to_phys (vmm_pd)    | PDPE_ATTR;
-	vmm_pdp[2] = sym_to_phys (vmm_pd1)   | PDPE_ATTR;
-	vmm_pdp[3] = 0;
-#ifdef __x86_64__
+	/* virt 0x40000000 vmm area */
+	vmm_pdp[1] = sym_to_phys (vmm_pd1)   | PDPE_ATTR;
+#ifndef __x86_64__
+	vmm_base_cr3 = sym_to_phys (vmm_pdp);
+#else
+	vmm_pml4[0] = sym_to_phys (vmm_pdp) |
+		PDE_P_BIT | PDE_RW_BIT | PDE_US_BIT;
 	vmm_base_cr3 = sym_to_phys (vmm_pml4);
-	vmm_pml4[0] = sym_to_phys (vmm_pdp) | PDE_P_BIT | PDE_RW_BIT |
-		PDE_US_BIT;
+#endif
+
+	/* make a new page directory for normal-use */
+
+	/* virt 0x40000000 vmm area */
+	vmm_main_pdp[1] = sym_to_phys (vmm_pd1) | PDPE_ATTR;
+	/* virt 0x80000000 mapmem area */
+	vmm_main_pdp[2] = sym_to_phys (vmm_pd2) | PDPE_ATTR;
+#ifndef __x86_64__
+	vmm_main_cr3 = sym_to_phys (vmm_main_pdp);
+#else
+	vmm_main_pml4[0] = sym_to_phys (vmm_main_pdp) |
+		PDE_P_BIT | PDE_RW_BIT | PDE_US_BIT;
+	vmm_main_cr3 = sym_to_phys (vmm_main_pml4);
 #endif
 
 #ifdef __x86_64__
@@ -273,6 +306,7 @@ move_vmm (void)
 #else
 	move_vmm_area32 ();
 #endif
+	asm_wrcr3(vmm_main_cr3);
 }
 
 static void
@@ -292,7 +326,7 @@ map_hphys (void)
 	}
 
 	memset (virt, 0, PAGESIZE);
-	vmm_pml4[1] = phys | PDE_P_BIT | PDE_RW_BIT | PDE_A_BIT;
+	vmm_main_pml4[1] = phys | PDE_P_BIT | PDE_RW_BIT | PDE_A_BIT;
 	pdp = virt;
 	pdpi = 0;
 	pdi = 512;
@@ -366,6 +400,7 @@ mm_init_global (void)
 		mm_page_free (&pagestruct[i]);
 	}
 	map_hphys ();
+	vramwrite_mm_initialized();
 }
 
 /* get a physical address of a symbol sym */
