@@ -666,9 +666,7 @@ vt_exit_reason (void)
 			 */
 			asm_vmxoff();
 			apic_send_init(get_apic_id());
-			for (;;) {
-				asm_cli_and_hlt ();
-			}
+			freeze();
 		} else {
 			vcpu_reset();
 			apic_enter_wait_for_sipi_state();
